#pragma rtGlobals=3		// Use modern global access method and strict wave access.

// Scroll Wheel Zoom, v0.8, H. Shiu, hjs4int@live.com
// Zoom in/out of graph with scroll wheel (both x and y simultaneously), centered around mouse cursor.
// Tries to zoom only relevant axes; should do ok with single plot graphs or graphs with plots
// in single row or column.  Might not do so well if there are multiple columns and rows (a matrix of
// plots).

// Activate on each graph or automatically enable on all _new_ graphs.
// Activate/automatically enable in Graph:Packages:Scroll Zoom menu or in Graph Marquee popup menu
// (is there a way to append to the Graph popup menu?).
// Can disable from control+right-mouse-button (Command+context-menu on Mac) popup.
// This pop-up also lists some helpful directions.
// Hold down shift for faster zooming.
// Hold down control (Command on Mac) to temporarily disable; this may be particularly useful
// for zooming single axes.



menu "Graph", dynamic
// Could apply check mark or dynamic menu, but this is good for now
	Submenu "Packages"
	Submenu "Scroll Wheel Zoom"
	scrollzoomstatus(returntype="check")+"Auto enable scroll wheel zooming on new graphs",/q, scrollzoomautotoggle() 
	"Enable scroll wheel zooming on top graph",/q, scrollzoomon()
	"Disable scroll wheel zooming on top graph",/q, scrollzoomoff()
	end
	end
end

menu "GraphMarquee"
// Could apply check mark or dynamic menu, but this is good for now
	"-"
	"Enable scroll wheel zooming", scrollzoomon()
	"Disable scroll wheel zooming", scrollzoomoff()
end

function scrollzoomon([wname])
	string wname
	
	if (paramisdefault(wname) == 1)
		wname = "kwTopWin"
	endif
	SetWindow $wname,hook(scrollzoom)=ScrollZoomHook
end

function scrollzoomoff()
	SetWindow kwTopWin,hook(scrollzoom)=$""
end

Menu "ScrollZoomDisablePopup", contextualmenu
	"Disable scroll wheel zooming", scrollzoomoff()
	"-"		// Sorry for the poor UX form, shouldn't mix info text with commands
	"(Directions<U", sleep/t(0)		// Formatting code (<U) not supported in Win
	"( Re-enable in Graph:Packages:Scroll Zoom menu or Graph Marquee pop-up menu", sleep/t(0)
	"( Hold down shift to zoom faster", sleep/t(0)
	"( Hold down control to temporarily disable, e.g., for single axis zooming", sleep/t(0)
	"( And, of course, control-click or Command-click for this menu", sleep/t(0)
End

// Hook to automatically enable Scroll Zoom on all _new_ graph windows.
function scrollzoom_autoon_newgraphs(wname, wtype)
//static function DISABLED_AfterWindowCreatedHook(wname, wtype)
	string wname
	variable wtype
	
	if (wtype == 1)
		scrollzoomon(wname=wname)
	endif
end

function/t scrollzoomstatus([returntype])
	string returntype
	string status = ""	// blank if off
	string szhookname = "scrollzoom_autoon_newgraphs"
	
	if (paramisdefault(returntype) == 1)
		returntype = ""
	endif
	
	setigorhook AfterWindowCreatedHook
	if (stringmatch(S_info, "*#"+szhookname+";*"))
		// Hook is on
		status = "on"
		if (stringmatch(returntype, "check"))
			status = "!"+num2char(18)
		else
			status = "on"
		endif
	endif
	
	//killstrings S_info
	return status
end

function scrollzoomautotoggle()
	string szhookname = "scrollzoom_autoon_newgraphs"
	string status
	
	status = scrollzoomstatus()
	
	if (strlen(status) > 0)
		setigorhook/k AfterWindowCreatedHook=$szhookname
	else
		setigorhook AfterWindowCreatedHook=$szhookname
	endif
end

Function ScrollZoomHook(s)
	STRUCT WMWinHookStruct &s
	string axes_t, axisname, info, dummy
	variable numaxes, i, j, f1, f2
	variable relx, rely, scale, mouseLoc, cursorval, rangeinc, newmax, newmin

	// bit 0:	Mouse button is down.
	//bit 1:	Shift key is down.
	//bit 2:	Option (Macintosh ) or Alt (Windows ) is down.
	//bit 3:	Command (Macintosh ) or Ctrl (Windows ) is down.
	//bit 4:	Contextual menu click: right-click or Control-click (Macintosh , or right-click (Windows ).
	switch(s.eventCode)
		case 3:		// Mouse down
		// Wish I could just append to the Graph context popup!
			if (((s.eventMod & 2^4) != 0) && ((s.eventMod & 2^3) != 0))		// Context menu button (right mouse button) and control pressed
				PopupContextualMenu/C=(s.mouseLoc.h, s.mouseLoc.v)/N "ScrollZoomDisablePopup"
			endif
			
			break
		case 22:		// Scroll wheel
			if ((s.eventMod & 2^3) == 0)		// Temp disable if control key held down

			axes_t = axislist("")
			numaxes = itemsinlist(axes_t)
			make/n=(numaxes)/free/o/t axes, sz_name, sz_type
			make/n=(numaxes)/free/o sz_relpos0, sz_relpos1, sz_min, sz_max
			
			getwindow kwTopWin, psizeDC
			rely = (s.mouseLoc.v - V_top) / (V_bottom - V_top)
			rely = 1 - rely
			relx = (s.mouseLoc.h - V_left) / (V_right - V_left)
	
			// Are we inside plot area?
			if ( (relx >= 0) && (relx <=1) && (rely >= 0) && (rely <=1))
			
			scale = 0.03
			scale *= s.wheelDy
			if ((s.eventMod & 2^1) != 0)		// Shift
				scale *= 8
			endif
			//if (s.wheelDy)
			//	scale = .03
			//	scale *= s.wheelDy
			//else
			//	scale = .25
			//	scale *= s.wheelDx
			//endif
			
			make/free/n=(numaxes)/t axes
			axes = stringfromlist(p, axes_t)
			// Get axis info and put into waves.  To do: move into sep routine.
			for (i=0; i<numaxes; i+=1)
				axisname = axes[i]
				info = axisinfo("", axisname)
				sz_name[i] = axisname
				sz_type[i] = stringbykey("AXTYPE", info)
				sscanf stringbykey("axisEnab(x)", info, "="), "{%f,%f}", f1, f2
				sz_relpos0[i] = f1
				sz_relpos1[i] = f2
				getaxis/q $axisname
				sz_min[i] = V_min
				sz_max[i] = V_max
			endfor

			// Check if we're within axis bounds
			extract/free/o/indx axes, haxes, (stringmatch(sz_type[p], "top") || stringmatch(sz_type[p], "bottom") )  && relx > sz_relpos0[p] && relx < sz_relpos1[p]
			extract/free/indx axes, vaxes, (stringmatch(sz_type[p], "left") || stringmatch(sz_type[p], "right") )  && rely > sz_relpos0[p] && rely < sz_relpos1[p]
			make/free/n=0 targetaxes
			concatenate/np {haxes, vaxes}, targetaxes

			pauseupdate; silent 1
			s.doSetCursor = 1
			s.cursorCode = 8
			for (i=0; i<numpnts(targetaxes); i+=1)
				j = targetaxes[i]
				if ( stringmatch(sz_type[j], "top") || stringmatch(sz_type[j], "bottom") )
					mouseLoc = s.mouseLoc.h
				else
					mouseLoc = s.mouseLoc.v
				endif
				axisname = axes[j]
				cursorval = axisvalfrompixel("", axisname, mouseLoc)
				rangeinc = (sz_max[j] - sz_min[j]) * scale
				newmax = sz_max[j] + -rangeinc * (sz_max[j] - cursorval) / (sz_max[j] - sz_min[j])
				newmin = sz_min[j] - -rangeinc * (cursorval - sz_min[j]) / (sz_max[j] - sz_min[j])
				SetAxis $axes[j] newmin, newmax
			endfor
			
			endif
			endif
			break
	EndSwitch
End